/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 * 
 * @author sven.jacobiATopen-xchange.com
 * 
 */

package com.openexchange.office.filter.odf;

import java.util.Iterator;

import org.apache.xerces.dom.ElementNSImpl;
import org.apache.xml.serializer.SerializationHandler;
import org.odftoolkit.odfdom.IElementWriter;
import org.odftoolkit.odfdom.pkg.OdfFileDom;
import org.xml.sax.SAXException;

public class MetaDataImpl extends ElementNSImpl implements IElementWriter {

	private static final long serialVersionUID = 1L;

	private DLList<MetaItem> content;

	public MetaDataImpl(OdfFileDom ownerDocument) {
		super(ownerDocument, Namespaces.OFFICE, "office:meta");
	}

	@Override
	public void writeObject(SerializationHandler output)
		throws SAXException {

        if(content!=null) {
    		SaxContextHandler.startElement(output, Namespaces.OFFICE, "meta", "office:meta");
    		final Iterator<MetaItem> contentIter = content.iterator();
    		while(contentIter.hasNext()) {
    			contentIter.next().writeObject(output);
    		}
    		SaxContextHandler.endElement(output, Namespaces.OFFICE, "meta", "office:meta");
        }
	}

	public MetaItem getItem(String qName) {
		final Iterator<MetaItem> contentIter = getContent().iterator();
		while(contentIter.hasNext()) {
			final MetaItem i = contentIter.next();
			if(qName.equals(i.getQName())) {
				return i;
			}
		}
		return null;
	}

	public void removeItem(String qName) {
		final Iterator<MetaItem> contentIter = getContent().iterator();
		while(contentIter.hasNext()) {
			final MetaItem i = contentIter.next();
			if(qName.equals(i.getQName())) {
				contentIter.remove();
				break;
			}
		}
	}

	public void setItem(MetaItem metaItem) {
		removeItem(metaItem.getQName());
		getContent().add(metaItem);
	}

	public DLList<MetaItem> getContent() {
        if(content==null) {
            content = new DLList<MetaItem>();
        }
        return content;
    }
}
