/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.filter.ooxml.pptx.operations;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import org.docx4j.dml.CTColorMappingOverride;
import org.docx4j.dml.CTRegularTextRun;
import org.docx4j.dml.CTTableStyle;
import org.docx4j.dml.CTTableStyleList;
import org.docx4j.dml.CTTextField;
import org.docx4j.dml.CTTextListStyle;
import org.docx4j.openpackaging.exceptions.InvalidFormatException;
import org.docx4j.openpackaging.packages.PresentationMLPackage;
import org.docx4j.openpackaging.parts.Part;
import org.docx4j.openpackaging.parts.PresentationML.JaxbPmlPart;
import org.docx4j.openpackaging.parts.PresentationML.MainPresentationPart;
import org.docx4j.openpackaging.parts.PresentationML.SlideLayoutPart;
import org.docx4j.openpackaging.parts.PresentationML.SlideMasterPart;
import org.docx4j.openpackaging.parts.PresentationML.SlidePart;
import org.docx4j.openpackaging.parts.PresentationML.TableStylesPart;
import org.docx4j.openpackaging.parts.PresentationML.ViewPropertiesPart;
import org.docx4j.openpackaging.parts.relationships.Namespaces;
import org.docx4j.openpackaging.parts.relationships.RelationshipsPart;
import org.docx4j.relationships.Relationship;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.pptx4j.pml.CTNormalViewPortion;
import org.pptx4j.pml.CTNormalViewProperties;
import org.pptx4j.pml.CTSlideMasterTextStyles;
import org.pptx4j.pml.Presentation;
import org.pptx4j.pml.Presentation.SldMasterIdLst;
import org.pptx4j.pml.Presentation.SldMasterIdLst.SldMasterId;
import org.pptx4j.pml.Presentation.SldSz;
import org.pptx4j.pml.Sld;
import org.pptx4j.pml.SldLayout;
import org.pptx4j.pml.SldMaster;
import org.pptx4j.pml.SlideLayoutIdList;
import org.pptx4j.pml.SlideLayoutIdList.SldLayoutId;
import org.pptx4j.pml.ViewPr;
import com.openexchange.office.filter.api.FilterException;
import com.openexchange.office.filter.api.OCKey;
import com.openexchange.office.filter.api.OCValue;
import com.openexchange.office.filter.ooxml.components.Component;
import com.openexchange.office.filter.ooxml.components.IShapeType;
import com.openexchange.office.filter.ooxml.drawingml.DMLHelper;
import com.openexchange.office.filter.ooxml.operations.CreateOperationHelper;
import com.openexchange.office.filter.ooxml.pptx.PptxOperationDocument;
import com.openexchange.office.filter.ooxml.pptx.components.HardBreakComponent;
import com.openexchange.office.filter.ooxml.pptx.components.ParagraphComponent;
import com.openexchange.office.filter.ooxml.pptx.components.RootComponent;
import com.openexchange.office.filter.ooxml.pptx.components.TableCellComponent;
import com.openexchange.office.filter.ooxml.pptx.components.TableRowComponent;
import com.openexchange.office.filter.ooxml.pptx.components.TextComponent;
import com.openexchange.office.filter.ooxml.pptx.components.TextFieldComponent;

public class PptxCreateOperationHelper extends CreateOperationHelper {

    private final PresentationMLPackage presentationMLPackage;

    // used only for layout ids ...
    private final HashMap<String, String> relTargetToId = new HashMap<String, String>();

    public PptxCreateOperationHelper(PptxOperationDocument _operationDocument, JSONArray mOperationsArray)
        throws FilterException {

        super(_operationDocument, mOperationsArray);
        presentationMLPackage = _operationDocument.getPackage();
    }

    @Override
    public PptxOperationDocument getOperationDocument() {
        return (PptxOperationDocument)operationDocument;
    }

    @Override
    public void createDocumentDefaults(String userLanguage)
        throws JSONException  {

        final MainPresentationPart mainPresentationPart = presentationMLPackage.getMainPresentationPart();
        final Presentation presentation = mainPresentationPart.getJaxbElement();
        final JSONObject documentAttributes = new JSONObject();

        // page settings
        final JSONObject jsonPageSettings = new JSONObject(3);
        final SldSz slideSz = presentation.getSldSz(true);
        jsonPageSettings.put(OCKey.WIDTH.value(), (int)((slideSz.getCx()/360)+0.5));
        jsonPageSettings.put(OCKey.HEIGHT.value(), (int)((slideSz.getCy()/360)+0.5));
        documentAttributes.put(OCKey.PAGE.value(), jsonPageSettings);

        // text default styles
        final CTTextListStyle defaultTextStyle = presentation.getDefaultTextStyle();
        if(defaultTextStyle!=null) {
            final JSONObject textDefaults = DMLHelper.createJsonFromTextListStyle(defaultTextStyle, mainPresentationPart);
            if(textDefaults!=null&&!textDefaults.isEmpty()) {
                final Object defaultCharacterAttributes = textDefaults.remove(OCKey.DEFAULT.value());
                if(!textDefaults.isEmpty()) {
                    documentAttributes.put(OCKey.DEFAULT_TEXT_LIST_STYLES.value(), textDefaults);
                }
                if(defaultCharacterAttributes instanceof JSONObject) {
                    final JSONObject language = ((JSONObject)defaultCharacterAttributes).optJSONObject(OCKey.CHARACTER.value());
                    if(language != null && language.optString(OCKey.LANGUAGE.value(), null) != null) {
                        documentAttributes.put(OCKey.CHARACTER.value(), language);
                    }
                }
            }
        }

        // layout settings
        final RelationshipsPart mainPresentationRelations = mainPresentationPart.getRelationshipsPart();
        final Relationship viewRel = mainPresentationRelations.getRelationshipByType(Namespaces.PRESENTATIONML_VIEW_PROPS);
        if(viewRel!=null) {
            final JSONObject layoutSettings = new JSONObject(1);
            final ViewPropertiesPart viewPropertiesPart = (ViewPropertiesPart)mainPresentationRelations.getPart(viewRel);
            final ViewPr viewPr = viewPropertiesPart.getJaxbElement();
            final CTNormalViewProperties normalViewProperties = viewPr.getNormalViewPr();
            if(normalViewProperties!=null) {
                final CTNormalViewPortion leftViewPortion = normalViewProperties.getRestoredLeft();
                if(leftViewPortion!=null) {
                    final double leftViewPortionSz = ((double)leftViewPortion.getSz()) / 1000;
                    layoutSettings.put(OCKey.SLIDE_PANE_WIDTH.value(), leftViewPortionSz);
                }
            }
            if(!layoutSettings.isEmpty()) {
                documentAttributes.put(OCKey.LAYOUT.value(), layoutSettings);
            }
        }

        final JSONObject setDocumentAttributesObject = new JSONObject(2);
        setDocumentAttributesObject.put(OCKey.NAME.value(), OCValue.SET_DOCUMENT_ATTRIBUTES.value());
        setDocumentAttributesObject.put(OCKey.ATTRS.value(), documentAttributes);
        operationsArray.put(setDocumentAttributesObject);
    }

    public void createMasterSlides()
        throws JSONException, ParseException, InvalidFormatException {

        final MainPresentationPart mainPresentationPart = presentationMLPackage.getMainPresentationPart();
        final SldMasterIdLst slideMasterIds = mainPresentationPart.getJaxbElement().getSldMasterIdLst();
        if(slideMasterIds!=null) {
            final List<SldMasterId> slideMasterIdList = slideMasterIds.getSldMasterId();
            for(int i=0; i<slideMasterIdList.size(); i++) {
                final SldMasterId slideMasterId = slideMasterIdList.get(i);
                final SlideMasterPart slideMasterPart = (SlideMasterPart)mainPresentationPart.getRelationshipsPart().getPart(slideMasterId.getRid());
                getOperationDocument().setContextPart(slideMasterPart);

                // context part is important to be able to access the correct theme
                if(i==0) {
                    createTableStyles();
                }

                final String masterTarget = slideMasterId.getId().toString();
                createMasterSlideOperations(slideMasterPart, masterTarget);

                // retrieving layouts
                final SlideLayoutIdList slideLayoutIds = slideMasterPart.getJaxbElement().getSldLayoutIdLst(false);
                if(slideLayoutIds!=null) {
                    final List<SldLayoutId> slideLayoutIdList = slideLayoutIds.getSldLayoutId();
                    for(int j=0; j<slideLayoutIdList.size(); j++) {
                        final SldLayoutId slideLayoutId = slideLayoutIdList.get(j);
                        relTargetToId.put(slideMasterPart.getRelationshipsPart().getRelationshipByID(slideLayoutId.getRid()).getTarget(), slideLayoutId.getId().toString());
                        createLayoutSlideOperations((SlideLayoutPart)slideMasterPart.getRelationshipsPart().getPart(slideLayoutId.getRid()), slideLayoutId.getId().toString(), masterTarget);
                    }
                }
                presentationMLPackage.getSourcePartStore().removeSourcePart(slideMasterPart);
                slideMasterPart.setJaxbElement(null);
            }
        }
    }

    public void createSlides()
        throws JSONException, ParseException {

        final RootComponent rootComponent = new RootComponent(getOperationDocument(), presentationMLPackage.getMainPresentationPart());
        Component slideComponent = rootComponent.getNextChildComponent(null, null);
        while (slideComponent!=null) {

            getOperationDocument().setContextPart((Part)slideComponent.getObject());

            final JSONObject jsonInsertSlideOperation = new JSONObject(4);
            jsonInsertSlideOperation.put(OCKey.NAME.value(), OCValue.INSERT_SLIDE.value());
            jsonInsertSlideOperation.put(OCKey.START.value(), slideComponent.getComponentNumber());
            final SlidePart slidePart = (SlidePart)slideComponent.getObject();

            // retrieving the layout target that has to be used for this slide
            final String slideLayoutTarget = slidePart.getRelationshipsPart().getRelationshipByType(Namespaces.PRESENTATIONML_SLIDE_LAYOUT).getTarget();
            if(slideLayoutTarget!=null) {
                final String slideLayoutTargetId = relTargetToId.get(slideLayoutTarget);
                if(slideLayoutTargetId!=null) {
                    jsonInsertSlideOperation.put(OCKey.TARGET.value(), slideLayoutTargetId);
                }
            }
            createCommonSlideDataOperations(slideComponent, slideComponent.getComponentNumber(), null, jsonInsertSlideOperation, new JSONObject());
            final JaxbPmlPart<?> part = ((JaxbPmlPart<?>)getOperationDocument().getContextPart());
            presentationMLPackage.getSourcePartStore().removeSourcePart(part);
            part.setJaxbElement(null);
            slideComponent = slideComponent.getNextComponent();
        }
    }

    public void createMasterSlideOperations(SlideMasterPart slideMasterPart, String target)
        throws JSONException, ParseException {

        final RootComponent rootComponent = new RootComponent(getOperationDocument(), slideMasterPart);
        final Component slideComponent = rootComponent.getNextChildComponent(null, null);
        if(slideComponent!=null) {
            final JSONObject jsonInsertMasterSlideOperation = new JSONObject(3);
            jsonInsertMasterSlideOperation.put(OCKey.NAME.value(), OCValue.INSERT_MASTER_SLIDE.value());
            jsonInsertMasterSlideOperation.put(OCKey.ID.value(), target);
            final SldMaster slideMaster = (SldMaster)(((JaxbPmlPart<?>)slideComponent.getObject()).getJaxbElement());
            final CTSlideMasterTextStyles txStyles = slideMaster.getTxStyles();
            final JSONObject attrs = new JSONObject();
            if(txStyles!=null) {
                final JSONObject styles = new JSONObject();
                final JSONObject titleStyle = DMLHelper.createJsonFromTextListStyle(txStyles.getTitleStyle(), (Part)slideComponent.getObject());
                if(titleStyle!=null&&!titleStyle.isEmpty()) {
                    styles.put(OCKey.TITLE.value(), titleStyle);
                }
                final JSONObject bodyStyle = DMLHelper.createJsonFromTextListStyle(txStyles.getBodyStyle(), (Part)slideComponent.getObject());
                if(bodyStyle!=null&&!bodyStyle.isEmpty()) {
                    styles.put(OCKey.BODY.value(), bodyStyle);
                }
                final JSONObject otherStyle = DMLHelper.createJsonFromTextListStyle(txStyles.getOtherStyle(), (Part)slideComponent.getObject());
                if(otherStyle!=null&&!otherStyle.isEmpty()) {
                    styles.put(OCKey.OTHER.value(), otherStyle);
                }
                if(!styles.isEmpty()) {
                    attrs.put(OCKey.LIST_STYLES.value(), styles);
                }
            }
            createCommonSlideDataOperations(slideComponent, 0, target, jsonInsertMasterSlideOperation, attrs);
        }
    }

    public void createLayoutSlideOperations(SlideLayoutPart slideLayoutPart, String target, String referencedMaster)
        throws JSONException, ParseException {

        final RootComponent rootComponent = new RootComponent(getOperationDocument(), slideLayoutPart);
        final Component slideComponent = rootComponent.getNextChildComponent(null, null);
        if(slideComponent!=null) {
            final JSONObject jsonInsertLayoutSlideOperation = new JSONObject(4);
            jsonInsertLayoutSlideOperation.put(OCKey.NAME.value(), OCValue.INSERT_LAYOUT_SLIDE.value());
            jsonInsertLayoutSlideOperation.put(OCKey.ID.value(), target);
            jsonInsertLayoutSlideOperation.put(OCKey.TARGET.value(), referencedMaster);
            createCommonSlideDataOperations(slideComponent, 0, target, jsonInsertLayoutSlideOperation, new JSONObject());
        }
        final JaxbPmlPart<?> part = ((JaxbPmlPart<?>)getOperationDocument().getContextPart());
        presentationMLPackage.getSourcePartStore().removeSourcePart(part);
        part.setJaxbElement(null);
    }

    public void createCommonSlideDataOperations(Component slideComponent, int slide, String target, JSONObject slideInsertOperation, JSONObject attrs)
        throws JSONException, ParseException {

        slideComponent.createJSONAttrs(attrs);

        final JaxbPmlPart<?> commonSlidePart = ((JaxbPmlPart<?>)slideComponent.getObject());
        final Object sldElement = commonSlidePart.getJaxbElement();

        if(!attrs.isEmpty()) {
            slideInsertOperation.put(OCKey.ATTRS.value(), attrs);
        }
        operationsArray.put(slideInsertOperation);

        if(sldElement instanceof SldMaster) {
            createThemeOperations(target, null, ((SldMaster)sldElement).getClrMap());
        }
        else if(sldElement instanceof SldLayout) {
            createClrMapOvrThemeOperation(commonSlidePart, target, null, ((SldLayout)sldElement).getClrMapOvr());
        }
        else if(sldElement instanceof Sld) {
            final JSONArray start = new JSONArray(1);
            start.put(slide);
            createClrMapOvrThemeOperation(commonSlidePart, target, start, ((Sld)sldElement).getClrMapOvr());
        }

        final ArrayList<Integer> position = new ArrayList<Integer>();
        position.add(slide);
        Component shapeComponent = slideComponent.getNextChildComponent(null, null);
        while(shapeComponent!=null) {
            createShapeOperations(shapeComponent, position, target);
            shapeComponent = shapeComponent.getNextComponent();
        }
    }

    public void createClrMapOvrThemeOperation(JaxbPmlPart<?> commonSlidePart, String target, JSONArray start, CTColorMappingOverride colorOverride)
        throws JSONException {

        if(colorOverride!=null) {
            if(colorOverride.getOverrideClrMapping()!=null) {
                createThemeOperations(target, start, colorOverride.getOverrideClrMapping());
            }
            else if(colorOverride.getMasterClrMapping()!=null) {
                // check if the parent  slideLayout and or master is using color mapping that is not default...
                if(commonSlidePart instanceof SlidePart) {
                    final SlideLayoutPart slideLayoutPart = ((SlidePart)commonSlidePart).getSlideLayoutPart();
                    if(slideLayoutPart!=null) {
                        // TODO: how is it possible to have an slide layout part without jaxbelement ?
                        if(slideLayoutPart.getJaxbElement()!=null) {
                            if(!slideLayoutPart.isDefaultColorMapping()) {
                                createThemeOperations(target, start, null);
                            }
                        }
                    }
                }
                else if (commonSlidePart instanceof SlideLayoutPart) {
                    if(!((SlideLayoutPart)commonSlidePart).isDefaultColorMapping()) {
                        createThemeOperations(target, start, null);
                    }
                }
            }
        }
    }

    public void createShapeOperations(Component shapeComponent, List<Integer> parentPosition, String target)
        throws JSONException, ParseException {

        final List<Integer> position = new ArrayList<Integer>(parentPosition);
        position.add(shapeComponent.getComponentNumber());

        final JSONObject jsonInsertOperation = new JSONObject(5);
        jsonInsertOperation.put(OCKey.NAME.value(), OCValue.INSERT_DRAWING.value());
        jsonInsertOperation.put(OCKey.TYPE.value(), ((IShapeType)shapeComponent).getType().toString());
        jsonInsertOperation.put(OCKey.START.value(), position);
        if(target!=null) {
            jsonInsertOperation.put(OCKey.TARGET.value(), target);
        }
        final JSONObject attrs = shapeComponent.createJSONAttrs(new JSONObject());
        if(!attrs.isEmpty()) {
            jsonInsertOperation.put(OCKey.ATTRS.value(), attrs);
        }
        operationsArray.put(jsonInsertOperation);
        Component shapeChildComponent = shapeComponent.getNextChildComponent(null, null);
        while(shapeChildComponent!=null) {
            if(shapeChildComponent instanceof ParagraphComponent) {
                createParagraphOperations((ParagraphComponent)shapeChildComponent, position, target);
            }
            else if(shapeChildComponent instanceof TableRowComponent) {
                createTableRowOperation((TableRowComponent)shapeChildComponent, position, target);
            }
            else {
                createShapeOperations(shapeChildComponent, position, target);
            }
            shapeChildComponent = shapeChildComponent.getNextComponent();
        }
    }

    public void createParagraphOperations(ParagraphComponent paragraphComponent, List<Integer> parentPosition, String target)
        throws JSONException {

        final List<Integer> position = new ArrayList<Integer>(parentPosition);
        position.add(paragraphComponent.getComponentNumber());
        final JSONObject jsonInsertParagraphOperation = new JSONObject(4);
        jsonInsertParagraphOperation.put(OCKey.NAME.value(), OCValue.INSERT_PARAGRAPH.value());
        jsonInsertParagraphOperation.put(OCKey.START.value(), position);
        if(target!=null) {
            jsonInsertParagraphOperation.put(OCKey.TARGET.value(), target);
        }
        final JSONObject attrs = paragraphComponent.createJSONAttrs(new JSONObject());
        if(!attrs.isEmpty()) {
            jsonInsertParagraphOperation.put(OCKey.ATTRS.value(), attrs);
        }
        operationsArray.put(jsonInsertParagraphOperation);
        Component paragraphChild = paragraphComponent.getNextChildComponent(null, null);
        while(paragraphChild!=null) {
            if(paragraphChild instanceof TextComponent) {
                int n = paragraphChild.getComponentNumber();
                final String[] tokens =
                    (((CTRegularTextRun)(((TextComponent)paragraphChild).getObject())).getT()).split("\\t", -1);
                for(int i=0; i < tokens.length; i++) {
                    final String t = tokens[i];
                    if(!t.isEmpty()) {
                        createTextInsertOperation(t, n, position, target);
                        n += t.length();
                    }
                    if(tokens.length!=1&&i!=tokens.length-1) {
                        createInsertTabOperation(n, position, target);
                        n++;
                    }
                }
            }
            else if(paragraphChild instanceof TextFieldComponent) {
                createInsertFieldOperation((TextFieldComponent)paragraphChild, position, target);
            }
            else if(paragraphChild instanceof HardBreakComponent) {
                createHardBreakOperation((HardBreakComponent)paragraphChild, position, target);
            }
            paragraphChild = paragraphChild.getNextComponent();
        }
        paragraphChild = paragraphComponent.getNextChildComponent(null, null);
        while(paragraphChild!=null) {
            if(paragraphChild instanceof TextComponent) {
                createTextAttributesOperation((TextComponent)paragraphChild, position, target);
            }
            else if(paragraphChild instanceof TextFieldComponent) {
                createTextFieldAttributesOperation((TextFieldComponent)paragraphChild, position, target);
            }
            else if(paragraphChild instanceof HardBreakComponent) {
                createHardBreakAttributesOperation((HardBreakComponent)paragraphChild, position, target);
            }
            paragraphChild = paragraphChild.getNextComponent();
        }
    }

    public void createTableRowOperation(TableRowComponent tableRowComponent, List<Integer> parentPosition, String target)
        throws JSONException {

        final List<Integer> position = new ArrayList<Integer>(parentPosition);
        position.add(tableRowComponent.getComponentNumber());
        final JSONObject jsonInsertRowOperation = new JSONObject(4);
        jsonInsertRowOperation.put(OCKey.NAME.value(), OCValue.INSERT_ROWS.value());
        jsonInsertRowOperation.put(OCKey.START.value(), position);
        if(target!=null) {
            jsonInsertRowOperation.put(OCKey.TARGET.value(), target);
        }
        final JSONObject attrs = tableRowComponent.createJSONAttrs(new JSONObject());
        if(!attrs.isEmpty()) {
            jsonInsertRowOperation.put(OCKey.ATTRS.value(), attrs);
        }
        operationsArray.put(jsonInsertRowOperation);

        Component tableRowChild = tableRowComponent.getNextChildComponent(null, null);
        while(tableRowChild!=null) {
            if(tableRowChild instanceof TableCellComponent) {
                createTableCellOperation((TableCellComponent)tableRowChild, position, target);
            }
            tableRowChild = tableRowChild.getNextComponent();
        }
    }

    public void createTableCellOperation(TableCellComponent tableCellComponent, List<Integer> parentPosition, String target)
        throws JSONException {

        final List<Integer> position = new ArrayList<Integer>(parentPosition);
        position.add(tableCellComponent.getComponentNumber());
        final JSONObject jsonInsertRowOperation = new JSONObject(4);
        jsonInsertRowOperation.put(OCKey.NAME.value(), OCValue.INSERT_CELLS.value());
        jsonInsertRowOperation.put(OCKey.START.value(), position);
        if(target!=null) {
            jsonInsertRowOperation.put(OCKey.TARGET.value(), target);
        }
        final JSONObject attrs = tableCellComponent.createJSONAttrs(new JSONObject());
        if(!attrs.isEmpty()) {
            jsonInsertRowOperation.put(OCKey.ATTRS.value(), attrs);
        }
        operationsArray.put(jsonInsertRowOperation);

        Component tableCellChild = tableCellComponent.getNextChildComponent(null, null);
        while(tableCellChild!=null) {
            if(tableCellChild instanceof ParagraphComponent) {
                createParagraphOperations((ParagraphComponent)tableCellChild, position, target);
            }
            tableCellChild = tableCellChild.getNextComponent();
        }
    }

    public void createTextInsertOperation(String text, int textPosition, List<Integer> parentPosition, String target)
        throws JSONException {

        final List<Integer> position = new ArrayList<Integer>(parentPosition);
        position.add(textPosition);

        final JSONObject jsonInsertTextOperation = new JSONObject(4);
        jsonInsertTextOperation.put(OCKey.NAME.value(), OCValue.INSERT_TEXT.value());
        jsonInsertTextOperation.put(OCKey.START.value(), position);
        if(target!=null) {
            jsonInsertTextOperation.put(OCKey.TARGET.value(), target);
        }
        jsonInsertTextOperation.put(OCKey.TEXT.value(), text);
        operationsArray.put(jsonInsertTextOperation);
    }

    public void createInsertTabOperation(int textPosition, List<Integer> parentPosition, String target)
        throws JSONException {

        final List<Integer> position = new ArrayList<Integer>(parentPosition);
        position.add(textPosition);

        final JSONObject jsonInsertTabOperation = new JSONObject(3);
        jsonInsertTabOperation.put(OCKey.NAME.value(), OCValue.INSERT_TAB.value());
        jsonInsertTabOperation.put(OCKey.START.value(), position);
        if(target!=null) {
            jsonInsertTabOperation.put(OCKey.TARGET.value(), target);
        }
        operationsArray.put(jsonInsertTabOperation);
    }

    public void createTextAttributesOperation(TextComponent textComponent, List<Integer> parentPosition, String target)
        throws JSONException {

        final JSONObject attrs = textComponent.createJSONAttrs(new JSONObject());
        if(!attrs.isEmpty()) {

            final JSONObject jsonSetAttributesOperation = new JSONObject(5);
            jsonSetAttributesOperation.put(OCKey.NAME.value(), OCValue.SET_ATTRIBUTES.value());

            final List<Integer> start = new ArrayList<Integer>(parentPosition);
            start.add(textComponent.getComponentNumber());
            jsonSetAttributesOperation.put(OCKey.START.value(), start);

            final CTRegularTextRun textRun = (CTRegularTextRun)textComponent.getObject();
            if(textRun.getT().length()>1) {
                start.set(start.size()-1, (textComponent.getComponentNumber()+textRun.getT().length())-1);
                jsonSetAttributesOperation.put(OCKey.END.value(), start);
            }

            if(target!=null) {
                jsonSetAttributesOperation.put(OCKey.TARGET.value(), target);
            }
            jsonSetAttributesOperation.put(OCKey.ATTRS.value(), attrs);
            operationsArray.put(jsonSetAttributesOperation);
        }
    }

    public void createHardBreakAttributesOperation(HardBreakComponent textComponent, List<Integer> parentPosition, String target)
        throws JSONException {

        final JSONObject attrs = textComponent.createJSONAttrs(new JSONObject());
        if(!attrs.isEmpty()) {

            final JSONObject jsonSetAttributesOperation = new JSONObject(5);
            jsonSetAttributesOperation.put(OCKey.NAME.value(), OCValue.SET_ATTRIBUTES.value());

            final List<Integer> start = new ArrayList<Integer>(parentPosition);
            start.add(textComponent.getComponentNumber());
            jsonSetAttributesOperation.put(OCKey.START.value(), start);
            if(target!=null) {
                jsonSetAttributesOperation.put(OCKey.TARGET.value(), target);
            }
            jsonSetAttributesOperation.put(OCKey.ATTRS.value(), attrs);
            operationsArray.put(jsonSetAttributesOperation);
        }
    }

    public void createTextFieldAttributesOperation(TextFieldComponent textFieldComponent, List<Integer> parentPosition, String target)
        throws JSONException {

        final JSONObject attrs = textFieldComponent.createJSONAttrs(new JSONObject());
        if(!attrs.isEmpty()) {

            final JSONObject jsonSetAttributesOperation = new JSONObject(4);
            jsonSetAttributesOperation.put(OCKey.NAME.value(), OCValue.SET_ATTRIBUTES.value());

            final List<Integer> start = new ArrayList<Integer>(parentPosition);
            start.add(textFieldComponent.getComponentNumber());
            jsonSetAttributesOperation.put(OCKey.START.value(), start);
            if(target!=null) {
                jsonSetAttributesOperation.put(OCKey.TARGET.value(), target);
            }
            jsonSetAttributesOperation.put(OCKey.ATTRS.value(), attrs);
            operationsArray.put(jsonSetAttributesOperation);
        }
    }

    public void createInsertFieldOperation(TextFieldComponent textFieldComponent, List<Integer> parentPosition, String target)
        throws JSONException {

        final List<Integer> position = new ArrayList<Integer>(parentPosition);
        position.add(textFieldComponent.getComponentNumber());

        final JSONObject jsonInsertSimpleFieldOperation = new JSONObject(6);
        jsonInsertSimpleFieldOperation.put(OCKey.NAME.value(), OCValue.INSERT_FIELD.value());
        jsonInsertSimpleFieldOperation.put(OCKey.START.value(), position);
        final CTTextField textField = (CTTextField)textFieldComponent.getObject();
        jsonInsertSimpleFieldOperation.put(OCKey.TYPE.value(), textField.getType());
        jsonInsertSimpleFieldOperation.put(OCKey.REPRESENTATION.value(), textField.getT());
        if(target!=null) {
            jsonInsertSimpleFieldOperation.put(OCKey.TARGET.value(), target);
        }
        final JSONObject attrs = textFieldComponent.createJSONAttrs(new JSONObject());
        if(!attrs.isEmpty()) {
            jsonInsertSimpleFieldOperation.put(OCKey.ATTRS.value(), attrs);
        }
        operationsArray.put(jsonInsertSimpleFieldOperation);
    }

    public void createHardBreakOperation(HardBreakComponent hardBreakComponent, List<Integer> parentPosition, String target)
        throws JSONException {

        final List<Integer> position = new ArrayList<Integer>(parentPosition);
        position.add(hardBreakComponent.getComponentNumber());

        final JSONObject jsonInsertHardBreakOperation = new JSONObject(4);
        jsonInsertHardBreakOperation.put(OCKey.NAME.value(), OCValue.INSERT_HARD_BREAK.value());
        jsonInsertHardBreakOperation.put(OCKey.START.value(), position);
        if(target!=null) {
            jsonInsertHardBreakOperation.put(OCKey.TARGET.value(), target);
        }
        final JSONObject attrs = hardBreakComponent.createJSONAttrs(new JSONObject());
        if(!attrs.isEmpty()) {
            jsonInsertHardBreakOperation.put(OCKey.ATTRS.value(), attrs);
        }
        operationsArray.put(jsonInsertHardBreakOperation);
    }

    public void createTableStyles()
        throws JSONException, InvalidFormatException {

        final TableStylesPart tableStylesPart = this.getOperationDocument().getTableStylesPart(false);
        if(tableStylesPart!=null) {
            final Object o = tableStylesPart.getJaxbElement();
            if(o instanceof CTTableStyleList) {
                final CTTableStyleList tableStyleList = (CTTableStyleList)o;
                final Iterator<CTTableStyle> tableStyleIter = tableStyleList.getTblStyle().iterator();
                while(tableStyleIter.hasNext()) {
                    final CTTableStyle tableStyle = tableStyleIter.next();
                    final String tableStyleId = tableStyle.getStyleId();
                    if(tableStyleId!=null&&!tableStyleId.isEmpty()) {
                        final JSONObject attrs = new JSONObject();
                        DMLHelper.createJsonFromTableStyle(attrs, tableStyle, getOperationDocument().getThemePart(false), tableStylesPart);
                        if(!attrs.isEmpty()) {
                            operationsArray.put(createInsertStyleSheetOperation("table", false, tableStyleId, tableStyle.getStyleName(), attrs, null, false, null, tableStyleId.equals(tableStyleList.getDef()), null));
                        }
                    }
                }
            }
        }
    }
}
